@@ -1,59 +1,16 @@
-/* ====================================================================
- * The Apache Software License, Version 1.1
+/* Copyright 1999-2004 The Apache Software Foundation
  *
- * Copyright (c) 2000-2002 The Apache Software Foundation.  All rights
- * reserved.
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
  *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
+ *     http://www.apache.org/licenses/LICENSE-2.0
  *
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- *
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in
- *    the documentation and/or other materials provided with the
- *    distribution.
- *
- * 3. The end-user documentation included with the redistribution,
- *    if any, must include the following acknowledgment:
- *       "This product includes software developed by the
- *        Apache Software Foundation (http://www.apache.org/)."
- *    Alternately, this acknowledgment may appear in the software itself,
- *    if and wherever such third-party acknowledgments normally appear.
- *
- * 4. The names "Apache" and "Apache Software Foundation" must
- *    not be used to endorse or promote products derived from this
- *    software without prior written permission. For written
- *    permission, please contact apache@apache.org.
- *
- * 5. Products derived from this software may not be called "Apache",
- *    nor may "Apache" appear in their name, without prior written
- *    permission of the Apache Software Foundation.
- *
- * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
- * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
- * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
- * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
- * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
- * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
- * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
- * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- * ====================================================================
- *
- * This software consists of voluntary contributions made by many
- * individuals on behalf of the Apache Software Foundation.  For more
- * information on the Apache Software Foundation, please see
- * <http://www.apache.org/>.
- *
- * Portions of this software are based upon public domain software
- * originally written at the National Center for Supercomputing Applications,
- * University of Illinois, Urbana-Champaign.
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
  */
 
 /*
@@ -291,7 +248,24 @@ static void cleanup_tables(void *not_used)
 }
 #endif	/* HAVE_SHMEM_MM */
 
-#ifdef WIN32
+#ifdef __OpenBSD__
+static void initialize_secret(server_rec *s)
+{
+    u_int32_t rnd = 0, i;
+
+    ap_log_error(APLOG_MARK, APLOG_NOERRNO|APLOG_NOTICE, s,
+		 "Digest: generating secret for digest authentication ...");
+
+    for (i = 0; i < sizeof(secret); i++) {
+	if (i % 4 == 0)
+	    rnd = arc4random();
+	secret[i] = rnd;
+	rnd >>= 8;
+    }
+    ap_log_error(APLOG_MARK, APLOG_NOERRNO|APLOG_NOTICE, s,
+		 "Digest: done");
+}
+#elif defined(WIN32)
 /* TODO: abstract out the random number generation. APR? */
 static void initialize_secret(server_rec *s)
 {
@@ -1155,76 +1129,10 @@ static void clear_session(const digest_header_rec *resp)
 	resp->client->ha1[0] = '\0';
 }
 
-
 /*
  * Authorization challenge generation code (for WWW-Authenticate)
  */
 
-static const char *guess_domain(pool *p, const char *uri, const char *filename,
-				const char *dir)
-{
-    size_t u_len = strlen(uri), f_len = strlen(filename), d_len = strlen(dir);
-    const char *u, *f;
-
-
-    /* Because of things like mod_alias and mod_rewrite and the fact that
-     * protection is often on a directory basis (not a location basis) it
-     * is hard to determine the uri to put in the domain attribute.
-     *
-     * What we do is the following: first we see if the directory is
-     * a prefix for the uri - if this is the case we assume that therefore
-     * a <Location> directive was protecting this uri and we can use it
-     * for the domain.
-     */
-    if (u_len >= d_len && !memcmp(uri, dir, d_len))
-	return dir;
-
-    /* Now we check for <Files ...>, and if we find one we send back a
-     * dummy uri - this is the only way to specify that the protection
-     * space only covers a single uri.
-     */
-    if (dir[0] != '/')
-	/* This doesn't work for Amaya (ok, it's of arguable validity in
-	 * the first place), so just return the file name instead
-	return "http://0.0.0.0/";
-	 */
-	return dir;
-
-    /* Next we find the largest common common suffix of the request-uri
-     * and the final file name, ignoring any extensions; this gives us a
-     * hint as to where any rewriting could've occured (assuming that some
-     * prefix of the uri is rewritten, not a suffix).
-     */
-    u = uri + u_len - 1;	/* strip any extension */
-    while (u > uri && *u != '/')  u--;
-    while (*u && *u != '.')  u++;
-    if (*u == '.')  u--;
-    if (*u == '/')  u--;
-
-    f = filename + f_len - 1;	/* strip any extension */
-    while (f > filename && *f != '/')  f--;
-    while (*f && *f != '.')  f++;
-    if (*f == '.')  f--;
-    if (*f == '/')  f--;
-
-    while (*f == *u && f > filename && u > uri)  u--, f--;
-    f++; u++;
-
-    while (*f && *f != '/')  f++, u++;	/* suffix must start with / */
-
-    /* Now, if the directory reaches into this common suffix then we can
-     * take the uri with the same reach.
-     */
-    if ((unsigned long) (f-filename) < d_len) {
-	char *tmp = ap_pstrdup(p, uri);
-	tmp[(u-uri)+(d_len-(f-filename))] = '\0';
-	return tmp;
-    }
-
-    return "";	/* give up */
-}
-
-
 static const char *ltox(pool *p, unsigned long num)
 {
     if (num != 0)
@@ -1306,18 +1214,15 @@ static void note_digest_auth_failure(request_rec *r,
      * unneccessarily (it's usually > 200 bytes!).
      */
 
-    if (r->proxyreq != NOT_PROXY)
-	domain = NULL;	/* don't send domain for proxy requests */
-    else if (conf->uri_list)
-	domain = conf->uri_list;
+    /* don't send domain
+     * - for proxy requests
+     * - if it's no specified
+     */
+    if (r->proxyreq || !conf->uri_list) {
+        domain = NULL;  
+    }
     else {
-	/* They didn't specify any domain, so let's guess at it */
-	domain = guess_domain(r->pool, resp->psd_request_uri->path, r->filename,
-			      conf->dir_name);
-	if (domain[0] == '/' && domain[1] == '\0')
-	    domain = NULL;	/* "/" is the default, so no need to send it */
-	else
-	    domain = ap_pstrcat(r->pool, ", domain=\"", domain, "\"", NULL);
+        domain = conf->uri_list;
     }
 
     ap_table_mergen(r->err_headers_out,
