@@ -1,59 +1,16 @@
-/* ====================================================================
- * The Apache Software License, Version 1.1
+/* Copyright 1999-2004 The Apache Software Foundation
  *
- * Copyright (c) 2000-2002 The Apache Software Foundation.  All rights
- * reserved.
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
  *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
+ *     http://www.apache.org/licenses/LICENSE-2.0
  *
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- *
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in
- *    the documentation and/or other materials provided with the
- *    distribution.
- *
- * 3. The end-user documentation included with the redistribution,
- *    if any, must include the following acknowledgment:
- *       "This product includes software developed by the
- *        Apache Software Foundation (http://www.apache.org/)."
- *    Alternately, this acknowledgment may appear in the software itself,
- *    if and wherever such third-party acknowledgments normally appear.
- *
- * 4. The names "Apache" and "Apache Software Foundation" must
- *    not be used to endorse or promote products derived from this
- *    software without prior written permission. For written
- *    permission, please contact apache@apache.org.
- *
- * 5. Products derived from this software may not be called "Apache",
- *    nor may "Apache" appear in their name, without prior written
- *    permission of the Apache Software Foundation.
- *
- * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
- * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
- * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
- * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
- * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
- * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
- * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
- * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- * ====================================================================
- *
- * This software consists of voluntary contributions made by many
- * individuals on behalf of the Apache Software Foundation.  For more
- * information on the Apache Software Foundation, please see
- * <http://www.apache.org/>.
- *
- * Portions of this software are based upon public domain software
- * originally written at the National Center for Supercomputing Applications,
- * University of Illinois, Urbana-Champaign.
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
  */
 
 /*
@@ -118,9 +75,11 @@
  * literal characters copied into the log files, and '%' directives as
  * follows:
  *
- * %...B:  bytes sent, excluding HTTP headers.
+ * %...a:  remote IP-address
+ * %...A:  local IP-address
  * %...b:  bytes sent, excluding HTTP headers in CLF format, i.e. a '-'
  *         when no bytes where sent (rather than a '0'.
+ * %...B:  bytes sent, excluding HTTP headers.
  * %...c:  Status of the connection.
  *         'X' = connection aborted before the response completed.
  *         '+' = connection may be kept alive after the response is sent.
@@ -128,15 +87,16 @@
  * %...{FOOBAR}e:  The contents of the environment variable FOOBAR
  * %...f:  filename
  * %...h:  remote host
- * %...a:  remote IP-address
- * %...A:  local IP-address
+ * %...H:  the request protocol
  * %...{Foobar}i:  The contents of Foobar: header line(s) in the request
  *                 sent to the client.
  * %...l:  remote logname (from identd, if supplied)
+ * %...m:  the request method
  * %...{Foobar}n:  The contents of note "Foobar" from another module.
  * %...{Foobar}o:  The contents of Foobar: header line(s) in the reply.
  * %...p:  the port the request was served to
  * %...P:  the process ID of the child that serviced the request.
+ * %...q:  the query string prepended by "?", or empty if no query string
  * %...r:  first line of request
  * %...s:  status.  For requests that got internally redirected, this
  *         is status of the *original* request --- %...>s for the last.
@@ -148,9 +108,7 @@
  * %...U:  the URL path requested.
  * %...v:  the configured name of the server (i.e. which virtual host?)
  * %...V:  the server name according to the UseCanonicalName setting
- * %...m:  the request method
- * %...H:  the request protocol
- * %...q:  the query string prepended by "?", or empty if no query string
+ * %...X:  An alias for %..c (Status of the connection).
  *
  * The '...' can be nothing at all (e.g. "%h %u %r %s %b"), or it can
  * indicate conditions for inclusion of the item (which will cause it
@@ -291,8 +249,8 @@ static const char *constant_item(request_rec *dummy, char *stuff)
 
 static const char *log_remote_host(request_rec *r, char *a)
 {
-    return ap_get_remote_host(r->connection, r->per_dir_config,
-                                    REMOTE_NAME);
+    return ap_escape_logitem(r->pool, ap_get_remote_host(r->connection, r->per_dir_config,
+                                    REMOTE_NAME));
 }
 
 static const char *log_remote_address(request_rec *r, char *a)
@@ -307,7 +265,7 @@ static const char *log_local_address(request_rec *r, char *a)
 
 static const char *log_remote_logname(request_rec *r, char *a)
 {
-    return ap_get_remote_logname(r);
+    return ap_escape_logitem(r->pool, ap_get_remote_logname(r));
 }
 
 static const char *log_remote_user(request_rec *r, char *a)
@@ -320,6 +278,8 @@ static const char *log_remote_user(request_rec *r, char *a)
     else if (strlen(rvalue) == 0) {
         rvalue = "\"\"";
     }
+    else
+        rvalue = ap_escape_logitem(r->pool, rvalue);
     return rvalue;
 }
 
@@ -330,10 +290,12 @@ static const char *log_request_line(request_rec *r, char *a)
 	     * (note the truncation before the protocol string for HTTP/0.9 requests)
 	     * (note also that r->the_request contains the unmodified request)
 	     */
-    return (r->parsed_uri.password) ? ap_pstrcat(r->pool, r->method, " ",
+    return ap_escape_logitem(r->pool,
+			     (r->parsed_uri.password) ? ap_pstrcat(r->pool, r->method, " ",
 					 ap_unparse_uri_components(r->pool, &r->parsed_uri, 0),
 					 r->assbackwards ? NULL : " ", r->protocol, NULL)
-					: r->the_request;
+					: r->the_request
+			     );
 }
 
 static const char *log_request_file(request_rec *r, char *a)
@@ -342,19 +304,20 @@ static const char *log_request_file(request_rec *r, char *a)
 }
 static const char *log_request_uri(request_rec *r, char *a)
 {
-    return r->uri;
+    return ap_escape_logitem(r->pool, r->uri);
 }
 static const char *log_request_method(request_rec *r, char *a)
 {
-    return r->method;
+    return ap_escape_logitem(r->pool, r->method);
 }
 static const char *log_request_protocol(request_rec *r, char *a)
 {
-    return r->protocol;
+    return ap_escape_logitem(r->pool, r->protocol);
 }
 static const char *log_request_query(request_rec *r, char *a)
 {
-    return (r->args != NULL) ? ap_pstrcat(r->pool, "?", r->args, NULL)
+    return (r->args != NULL) ? ap_pstrcat(r->pool, "?",
+					  ap_escape_logitem(r->pool, r->args), NULL)
                              : "";
 }
 static const char *log_status(request_rec *r, char *a)
@@ -389,7 +352,7 @@ static const char *log_bytes_sent(request_rec *r, char *a)
 
 static const char *log_header_in(request_rec *r, char *a)
 {
-    return ap_table_get(r->headers_in, a);
+    return ap_escape_logitem(r->pool, ap_table_get(r->headers_in, a));
 }
 
 static const char *log_header_out(request_rec *r, char *a)
@@ -470,6 +433,7 @@ static const char *log_child_pid(request_rec *r, char *a)
 {
     return ap_psprintf(r->pool, "%ld", (long) getpid());
 }
+
 static const char *log_connection_status(request_rec *r, char *a)
 {
     if (r->connection->aborted)
@@ -482,6 +446,7 @@ static const char *log_connection_status(request_rec *r, char *a)
 
     return "-";
 }
+
 /*****************************************************************
  *
  * Parsing the log format string
@@ -493,39 +458,12 @@ static struct log_item_list {
     int want_orig_default;
 } log_item_keys[] = {
 
-    {
-        'h', log_remote_host, 0
-    },
     {   
         'a', log_remote_address, 0 
     },
     {   
         'A', log_local_address, 0 
     },
-    {
-        'l', log_remote_logname, 0
-    },
-    {
-        'u', log_remote_user, 0
-    },
-    {
-        't', log_request_time, 0
-    },
-    {
-        'T', log_request_duration, 1
-    },
-    {
-        'r', log_request_line, 1
-    },
-    {
-        'f', log_request_file, 0
-    },
-    {
-        'U', log_request_uri, 1
-    },
-    {
-        's', log_status, 1
-    },
     {
         'b', clf_log_bytes_sent, 0
     },
@@ -533,22 +471,34 @@ static struct log_item_list {
         'B', log_bytes_sent, 0
     },
     {
-        'i', log_header_in, 0
-    },
-    {
-        'o', log_header_out, 0
-    },
-    {
-        'n', log_note, 0
+        'c', log_connection_status, 0
     },
     {
         'e', log_env_var, 0
     },
     {
-        'V', log_server_name, 0
+        'f', log_request_file, 0
     },
     {
-        'v', log_virtual_host, 0
+        'h', log_remote_host, 0
+    },
+    {
+        'H', log_request_protocol, 0
+    },
+    {
+        'i', log_header_in, 0
+    },
+    {
+        'l', log_remote_logname, 0
+    },
+    {
+        'm', log_request_method, 0
+    },
+    {
+        'n', log_note, 0
+    },
+    {
+        'o', log_header_out, 0
     },
     {
         'p', log_server_port, 0
@@ -556,17 +506,35 @@ static struct log_item_list {
     {
         'P', log_child_pid, 0
     },
-    {
-        'H', log_request_protocol, 0
-    },
-    {
-        'm', log_request_method, 0
-    },
     {
         'q', log_request_query, 0
     },
     {
-        'c', log_connection_status, 0
+        'r', log_request_line, 1
+    },
+    {
+        's', log_status, 1
+    },
+    {
+        't', log_request_time, 0
+    },
+    {
+        'T', log_request_duration, 1
+    },
+    {
+        'u', log_remote_user, 0
+    },
+    {
+        'U', log_request_uri, 1
+    },
+    {
+        'v', log_virtual_host, 0
+    },
+    {
+        'V', log_server_name, 0
+    },
+    {
+        'X', log_connection_status, 0
     },
     {
         '\0'
@@ -1062,7 +1030,8 @@ static config_log_state *open_config_log(server_rec *s, pool *p,
     }
     else {
         char *fname = ap_server_root_relative(p, cls->fname);
-        if ((cls->log_fd = ap_popenf(p, fname, xfer_flags, xfer_mode)) < 0) {
+        if ((cls->log_fd = ap_popenf_ex(p, fname, xfer_flags, xfer_mode, 1))
+             < 0) {
             ap_log_error(APLOG_MARK, APLOG_ERR, s,
                          "could not open transfer log file %s.", fname);
             exit(1);
